;; Copyright (C) 2024  Felisp
;;
;; This program is free software: you can redistribute it and/or modify
;; it under the terms of the GNU Affero General Public License as published by
;; the Free Software Foundation, version 3 of the License.
;;
;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU Affero General Public License for more details.
;;
;; You should have received a copy of the GNU Affero General Public License
;; along with this program.  If not, see <https://www.gnu.org/licenses/>.

(ns rss-thread-watch.core
  (:require [clojure.tools.cli :refer [parse-opts]]
            [ring.adapter.jetty :as jetty]
            [ring.middleware.params :as rp]
            [rss-thread-watch.watcher :as watcher]
            [rss-thread-watch.feed-generator :as feed]
            [rss-thread-watch.utils :as u]
            [rss-thread-watch.config :as conf])
  (:gen-class))


(def cli-options
  "Configuration defining program arguments for cli.tools"
  [["-v" "--version" "Print version and license information"]
   ["-h" "--help" "Prints help"]
   ["-c" "--config CONFIG_FILE" "Specify config file to use for this run"
    :default "./config.edn"
    :validate [#(u/file-exists? %) "Specified config file does not exist or is not readable"]]
   [nil "--print-default-config" "Prints internal default config file to STDOUT and exits"]])

;; Todo: Think of a way to start repeated download for every catalog efficiently
(defn set-interval
  "Calls function every ms"
  ^{:deprecated true}
  [callback ms]
  (future (while true (do (try
                            (callback)
                            (println "Recached")
                            (catch Exception e
                              (binding [*out* *err*]
                                (println "Error while updating cache: " e ", retrying in " (/ ms 1000 60) " minutes"))))
                          (Thread/sleep ms)))))

(defn -main
  "Entry point, starts webserver"
  [& args]
  (let [parsed-args (parse-opts args cli-options)
        options (get parsed-args :options)]
    (when-let [err (get parsed-args :errors)]
      (println "Error: " err)
      (System/exit 1))
    (when (get options :version)
      (println "RSS Thread Watcher " conf/VERSION " Licensed under AGPL-3.0-only")
      (System/exit 0))
    (when (get options :help)
      (println "RSS Thread Watcher help:\n" (get parsed-args :summary))
      (System/exit 0))
    (when (get options :print-default-config)
      (println ";;Default internal config file from RSS Thread Watcher " conf/VERSION)
      (clojure.pprint/pprint conf/CONFIG-DEFAULT)
      ;; In case someone was copying by hand, this might be useful
      (println ";;END of Default internal config file")
      (System/exit 0))

    (let [config (conf/get-some-config (:config options))]
      ;; TODO: probably refactor to use separate config.clj file when validation will be added
      ;; Init the few globals we have
      ;; TODO: this all needs to go in separate function so it doesnt have to duplicated in repl-main
      (reset! conf/GLOBAL-CONFIG config)
      (reset! feed/boards-enabled-cache (set (keys (get config :boards-enabled))))
      (reset! watcher/chod-threads-cache (watcher/generate-chod-cache-structure config))
      (clojure.pprint/pprint config)
      (jetty/run-jetty (rp/wrap-params feed/http-handler) {:port (:port config)
                                                           :join? true}))))

;; Docs: https://github.com/ring-clojure/ring/wiki/Getting-Started
(defn repl-main
  "Development entry point"
  []
  (let [config (conf/get-some-config nil)]
      ;; TODO: probably refactor to use separate config.clj file when validation will be added
      ;; Init the few globals we have
      (reset! conf/GLOBAL-CONFIG config)
      (reset! feed/boards-enabled-cache (set (keys (get config :boards-enabled))))
      (reset! watcher/chod-threads-cache (watcher/generate-chod-cache-structure config)))
  (jetty/run-jetty (rp/wrap-params #'feed/http-handler)
                   {:port (:port conf/CONFIG-DEFAULT)
                    ;; Dont block REPL thread
                    :join? false}))
;; (repl-main)
